/*
 * Automatically generated C config: don't edit
 */
#define AUTOCONF_INCLUDED

/*
 * Code maturity level options
 */
#define CONFIG_EXPERIMENTAL 1

/*
 * Processor type and features
 */
#define CONFIG_M386 1
#undef  CONFIG_M486
#undef  CONFIG_M586
#undef  CONFIG_M586TSC
#undef  CONFIG_M686
#define CONFIG_MATH_EMULATION 1
#define CONFIG_MTRR 1
#undef  CONFIG_SMP

/*
 * Loadable module support
 */
#define CONFIG_MODULES 1
#undef  CONFIG_MODVERSIONS
#undef  CONFIG_KMOD

/*
 * General setup
 */
#define CONFIG_NET 1
#define CONFIG_PCI 1
#undef  CONFIG_PCI_GOBIOS
#undef  CONFIG_PCI_GODIRECT
#define CONFIG_PCI_GOANY 1
#define CONFIG_PCI_BIOS 1
#define CONFIG_PCI_DIRECT 1
#define CONFIG_PCI_QUIRKS 1
#undef  CONFIG_PCI_OPTIMIZE
#define CONFIG_PCI_OLD_PROC 1
#undef  CONFIG_MCA
#undef  CONFIG_VISWS
#define CONFIG_SYSVIPC 1
#define CONFIG_BSD_PROCESS_ACCT 1
#define CONFIG_SYSCTL 1
#define CONFIG_BINFMT_AOUT 1
#define CONFIG_BINFMT_ELF 1
#define CONFIG_BINFMT_MISC 1
#undef  CONFIG_BINFMT_JAVA
#define CONFIG_BINFMT_JAVA_MODULE 1
#undef  CONFIG_PARPORT
#define CONFIG_PARPORT_MODULE 1
#undef  CONFIG_PARPORT_PC
#define CONFIG_PARPORT_PC_MODULE 1
#undef  CONFIG_PARPORT_OTHER
#undef  CONFIG_APM

/*
 * Plug and Play support
 */
#define CONFIG_PNP 1
#undef  CONFIG_PNP_PARPORT
#define CONFIG_PNP_PARPORT_MODULE 1

/*
 * Block devices
 */
#define CONFIG_BLK_DEV_FD 1
#define CONFIG_BLK_DEV_IDE 1

/*
 * Please see Documentation/ide.txt for help/info on IDE drives
 */
#undef  CONFIG_BLK_DEV_HD_IDE
#define CONFIG_BLK_DEV_IDEDISK 1
#define CONFIG_BLK_DEV_IDECD 1
#undef  CONFIG_BLK_DEV_IDETAPE
#define CONFIG_BLK_DEV_IDETAPE_MODULE 1
#define CONFIG_BLK_DEV_IDEFLOPPY 1
#undef  CONFIG_BLK_DEV_IDESCSI
#define CONFIG_BLK_DEV_CMD640 1
#undef  CONFIG_BLK_DEV_CMD640_ENHANCED
#define CONFIG_BLK_DEV_RZ1000 1
#define CONFIG_BLK_DEV_IDEPCI 1
#define CONFIG_BLK_DEV_IDEDMA 1
#undef  CONFIG_BLK_DEV_OFFBOARD
#define CONFIG_IDEDMA_AUTO 1
#undef  CONFIG_BLK_DEV_OPTI621
#undef  CONFIG_BLK_DEV_TRM290
#undef  CONFIG_BLK_DEV_NS87415
#undef  CONFIG_BLK_DEV_VIA82C586
#define CONFIG_BLK_DEV_CMD646 1
#define CONFIG_IDE_CHIPSETS 1

/*
 * Note: most of these also require special kernel boot parameters
 */
#define CONFIG_BLK_DEV_4DRIVES 1
#define CONFIG_BLK_DEV_ALI14XX 1
#define CONFIG_BLK_DEV_DTC2278 1
#define CONFIG_BLK_DEV_HT6560B 1
#define CONFIG_BLK_DEV_QD6580 1
#define CONFIG_BLK_DEV_UMC8672 1
#define CONFIG_BLK_DEV_PDC4030 1

/*
 * Additional Block Devices
 */
#define CONFIG_BLK_DEV_LOOP 1
#define CONFIG_BLK_DEV_NBD 1
#define CONFIG_BLK_DEV_MD 1
#define CONFIG_MD_LINEAR 1
#define CONFIG_MD_STRIPED 1
#undef  CONFIG_MD_MIRRORING
#define CONFIG_MD_MIRRORING_MODULE 1
#undef  CONFIG_MD_RAID5
#define CONFIG_MD_RAID5_MODULE 1
#define CONFIG_MD_BOOT 1
#define CONFIG_BLK_DEV_RAM 1
#define CONFIG_BLK_DEV_INITRD 1
#undef  CONFIG_BLK_DEV_XD
#define CONFIG_BLK_DEV_XD_MODULE 1
#undef  CONFIG_PARIDE_PARPORT
#define CONFIG_PARIDE_PARPORT_MODULE 1
#undef  CONFIG_PARIDE
#define CONFIG_PARIDE_MODULE 1

/*
 * Parallel IDE high-level drivers
 */
#undef  CONFIG_PARIDE_PD
#define CONFIG_PARIDE_PD_MODULE 1
#undef  CONFIG_PARIDE_PCD
#define CONFIG_PARIDE_PCD_MODULE 1
#undef  CONFIG_PARIDE_PF
#define CONFIG_PARIDE_PF_MODULE 1
#undef  CONFIG_PARIDE_PT
#define CONFIG_PARIDE_PT_MODULE 1
#undef  CONFIG_PARIDE_PG
#define CONFIG_PARIDE_PG_MODULE 1

/*
 * Parallel IDE protocol modules
 */
#undef  CONFIG_PARIDE_ATEN
#define CONFIG_PARIDE_ATEN_MODULE 1
#undef  CONFIG_PARIDE_BPCK
#define CONFIG_PARIDE_BPCK_MODULE 1
#undef  CONFIG_PARIDE_COMM
#define CONFIG_PARIDE_COMM_MODULE 1
#undef  CONFIG_PARIDE_DSTR
#define CONFIG_PARIDE_DSTR_MODULE 1
#undef  CONFIG_PARIDE_FIT2
#define CONFIG_PARIDE_FIT2_MODULE 1
#undef  CONFIG_PARIDE_FIT3
#define CONFIG_PARIDE_FIT3_MODULE 1
#undef  CONFIG_PARIDE_EPAT
#define CONFIG_PARIDE_EPAT_MODULE 1
#undef  CONFIG_PARIDE_EPIA
#define CONFIG_PARIDE_EPIA_MODULE 1
#undef  CONFIG_PARIDE_FRIQ
#define CONFIG_PARIDE_FRIQ_MODULE 1
#undef  CONFIG_PARIDE_FRPW
#define CONFIG_PARIDE_FRPW_MODULE 1
#undef  CONFIG_PARIDE_KBIC
#define CONFIG_PARIDE_KBIC_MODULE 1
#undef  CONFIG_PARIDE_KTTI
#define CONFIG_PARIDE_KTTI_MODULE 1
#undef  CONFIG_PARIDE_ON20
#define CONFIG_PARIDE_ON20_MODULE 1
#undef  CONFIG_PARIDE_ON26
#define CONFIG_PARIDE_ON26_MODULE 1
#undef  CONFIG_BLK_DEV_HD

/*
 * Networking options
 */
#define CONFIG_PACKET 1
#define CONFIG_NETLINK 1
#define CONFIG_RTNETLINK 1
#undef  CONFIG_NETLINK_DEV
#define CONFIG_NETLINK_DEV_MODULE 1
#define CONFIG_FIREWALL 1
#undef  CONFIG_FILTER
#define CONFIG_UNIX 1
#define CONFIG_INET 1
#define CONFIG_IP_MULTICAST 1
#undef  CONFIG_IP_ADVANCED_ROUTER
#undef  CONFIG_IP_PNP
#define CONFIG_IP_FIREWALL 1
#undef  CONFIG_IP_FIREWALL_NETLINK
#define CONFIG_IP_ALWAYS_DEFRAG 1
#define CONFIG_IP_TRANSPARENT_PROXY 1
#define CONFIG_IP_MASQUERADE 1

/*
 * Protocol-specific masquerading support will be built as modules.
 */
#define CONFIG_IP_MASQUERADE_ICMP 1

/*
 * Protocol-specific masquerading support will be built as modules.
 */
#define CONFIG_IP_MASQUERADE_MOD 1
#undef  CONFIG_IP_MASQUERADE_IPAUTOFW
#define CONFIG_IP_MASQUERADE_IPAUTOFW_MODULE 1
#undef  CONFIG_IP_MASQUERADE_IPPORTFW
#define CONFIG_IP_MASQUERADE_IPPORTFW_MODULE 1
#undef  CONFIG_IP_MASQUERADE_MFW
#define CONFIG_IP_MASQUERADE_MFW_MODULE 1
#undef  CONFIG_IP_ROUTER
#undef  CONFIG_NET_IPIP
#define CONFIG_NET_IPIP_MODULE 1
#undef  CONFIG_NET_IPGRE
#define CONFIG_NET_IPGRE_MODULE 1
#undef  CONFIG_NET_IPGRE_BROADCAST
#undef  CONFIG_IP_MROUTE
#undef  CONFIG_IP_ALIAS
#undef  CONFIG_ARPD
#define CONFIG_SYN_COOKIES 1

/*
 * (it is safe to leave these untouched)
 */
#undef  CONFIG_INET_RARP
#define CONFIG_INET_RARP_MODULE 1
#undef  CONFIG_SKB_LARGE
#undef  CONFIG_IPV6
#define CONFIG_IPV6_MODULE 1
#define CONFIG_IPV6_EUI64 1
#define CONFIG_IPV6_NO_PB 1

/*
 *  
 */
#undef  CONFIG_IPX
#define CONFIG_IPX_MODULE 1
#undef  CONFIG_IPX_INTERN
#undef  CONFIG_SPX
#define CONFIG_SPX_MODULE 1
#undef  CONFIG_ATALK
#define CONFIG_ATALK_MODULE 1
#undef  CONFIG_X25
#define CONFIG_X25_MODULE 1
#undef  CONFIG_LAPB
#define CONFIG_LAPB_MODULE 1
#undef  CONFIG_BRIDGE
#undef  CONFIG_LLC
#undef  CONFIG_ECONET
#define CONFIG_ECONET_MODULE 1
#undef  CONFIG_ECONET_AUNUDP
#undef  CONFIG_ECONET_NATIVE
#undef  CONFIG_WAN_ROUTER
#define CONFIG_WAN_ROUTER_MODULE 1
#undef  CONFIG_NET_FASTROUTE
#undef  CONFIG_NET_HW_FLOWCONTROL
#undef  CONFIG_CPU_IS_SLOW

/*
 * QoS and/or fair queueing
 */
#undef  CONFIG_NET_SCHED

/*
 * SCSI support
 */
#define CONFIG_SCSI 1

/*
 * SCSI support type (disk, tape, CD-ROM)
 */
#define CONFIG_BLK_DEV_SD 1
#define CONFIG_CHR_DEV_ST 1
#define CONFIG_BLK_DEV_SR 1
#undef  CONFIG_BLK_DEV_SR_VENDOR
#define CONFIG_CHR_DEV_SG 1

/*
 * Some SCSI devices (e.g. CD jukebox) support multiple LUNs
 */
#undef  CONFIG_SCSI_MULTI_LUN
#define CONFIG_SCSI_CONSTANTS 1
#define CONFIG_SCSI_LOGGING 1

/*
 * SCSI low-level drivers
 */
#undef  CONFIG_SCSI_7000FASST
#define CONFIG_SCSI_7000FASST_MODULE 1
#undef  CONFIG_SCSI_ACARD
#define CONFIG_SCSI_ACARD_MODULE 1
#undef  CONFIG_SCSI_AHA152X
#define CONFIG_SCSI_AHA152X_MODULE 1
#undef  CONFIG_SCSI_AHA1542
#define CONFIG_SCSI_AHA1542_MODULE 1
#undef  CONFIG_SCSI_AHA1740
#define CONFIG_SCSI_AHA1740_MODULE 1
#undef  CONFIG_SCSI_AIC7XXX
#define CONFIG_SCSI_AIC7XXX_MODULE 1
#undef  CONFIG_OVERRIDE_CMDS
#undef  CONFIG_AIC7XXX_PROC_STATS
#define CONFIG_AIC7XXX_RESET_DELAY (5)
#undef  CONFIG_SCSI_ADVANSYS
#define CONFIG_SCSI_ADVANSYS_MODULE 1
#undef  CONFIG_SCSI_IN2000
#define CONFIG_SCSI_IN2000_MODULE 1
#undef  CONFIG_SCSI_AM53C974
#define CONFIG_SCSI_AM53C974_MODULE 1
#undef  CONFIG_SCSI_MEGARAID
#define CONFIG_SCSI_MEGARAID_MODULE 1
#define CONFIG_MEGARAID_MULTI_IO 1
#undef  CONFIG_SCSI_BUSLOGIC
#define CONFIG_SCSI_BUSLOGIC_MODULE 1
#undef  CONFIG_SCSI_OMIT_FLASHPOINT
#undef  CONFIG_SCSI_DTC3280
#define CONFIG_SCSI_DTC3280_MODULE 1
#undef  CONFIG_SCSI_EATA
#define CONFIG_SCSI_EATA_MODULE 1
#undef  CONFIG_SCSI_EATA_TAGGED_QUEUE
#undef  CONFIG_SCSI_EATA_LINKED_COMMANDS
#define CONFIG_SCSI_EATA_MAX_TAGS (16)
#undef  CONFIG_SCSI_EATA_DMA
#define CONFIG_SCSI_EATA_DMA_MODULE 1
#undef  CONFIG_SCSI_EATA_PIO
#define CONFIG_SCSI_EATA_PIO_MODULE 1
#undef  CONFIG_SCSI_FUTURE_DOMAIN
#define CONFIG_SCSI_FUTURE_DOMAIN_MODULE 1
#undef  CONFIG_SCSI_GDTH
#define CONFIG_SCSI_GDTH_MODULE 1
#undef  CONFIG_SCSI_GENERIC_NCR5380
#define CONFIG_SCSI_GENERIC_NCR5380_MODULE 1
#define CONFIG_SCSI_GENERIC_NCR53C400 1
#define CONFIG_SCSI_G_NCR5380_PORT 1
#undef  CONFIG_SCSI_G_NCR5380_MEM
#undef  CONFIG_SCSI_INITIO
#define CONFIG_SCSI_INITIO_MODULE 1
#undef  CONFIG_SCSI_INIA100
#define CONFIG_SCSI_INIA100_MODULE 1
#undef  CONFIG_SCSI_PPA
#define CONFIG_SCSI_PPA_MODULE 1
#undef  CONFIG_SCSI_IMM
#define CONFIG_SCSI_IMM_MODULE 1
#define CONFIG_SCSI_IZIP_EPP16 1
#undef  CONFIG_SCSI_IZIP_SLOW_CTR
#undef  CONFIG_SCSI_NCR53C406A
#define CONFIG_SCSI_NCR53C406A_MODULE 1
#undef  CONFIG_SCSI_SYM53C416
#define CONFIG_SCSI_SYM53C416_MODULE 1
#undef  CONFIG_SCSI_NCR53C7xx
#define CONFIG_SCSI_NCR53C7xx_MODULE 1
#undef  CONFIG_SCSI_NCR53C7xx_sync
#undef  CONFIG_SCSI_NCR53C7xx_FAST
#undef  CONFIG_SCSI_NCR53C7xx_DISCONNECT
#undef  CONFIG_SCSI_NCR53C8XX
#define CONFIG_SCSI_NCR53C8XX_MODULE 1
#undef  CONFIG_SCSI_SYM53C8XX
#define CONFIG_SCSI_SYM53C8XX_MODULE 1
#define CONFIG_SCSI_NCR53C8XX_DEFAULT_TAGS (8)
#define CONFIG_SCSI_NCR53C8XX_MAX_TAGS (32)
#define CONFIG_SCSI_NCR53C8XX_SYNC (20)
#undef  CONFIG_SCSI_NCR53C8XX_PROFILE
#undef  CONFIG_SCSI_NCR53C8XX_IOMAPPED
#define CONFIG_SCSI_NCR53C8XX_PQS_PDS 1
#undef  CONFIG_SCSI_NCR53C8XX_SYMBIOS_COMPAT
#undef  CONFIG_SCSI_PAS16
#define CONFIG_SCSI_PAS16_MODULE 1
#undef  CONFIG_SCSI_PCI2000
#define CONFIG_SCSI_PCI2000_MODULE 1
#undef  CONFIG_SCSI_PCI2220I
#define CONFIG_SCSI_PCI2220I_MODULE 1
#undef  CONFIG_SCSI_PSI240I
#define CONFIG_SCSI_PSI240I_MODULE 1
#undef  CONFIG_SCSI_QLOGIC_FAS
#define CONFIG_SCSI_QLOGIC_FAS_MODULE 1
#undef  CONFIG_SCSI_QLOGIC_ISP
#define CONFIG_SCSI_QLOGIC_ISP_MODULE 1
#undef  CONFIG_SCSI_QLOGIC_FC
#define CONFIG_SCSI_QLOGIC_FC_MODULE 1
#undef  CONFIG_SCSI_SEAGATE
#define CONFIG_SCSI_SEAGATE_MODULE 1
#undef  CONFIG_SCSI_DC390T
#define CONFIG_SCSI_DC390T_MODULE 1
#undef  CONFIG_SCSI_DC390T_NOGENSUPP
#undef  CONFIG_SCSI_T128
#define CONFIG_SCSI_T128_MODULE 1
#undef  CONFIG_SCSI_U14_34F
#define CONFIG_SCSI_U14_34F_MODULE 1
#undef  CONFIG_SCSI_U14_34F_LINKED_COMMANDS
#define CONFIG_SCSI_U14_34F_MAX_TAGS (8)
#undef  CONFIG_SCSI_ULTRASTOR
#define CONFIG_SCSI_ULTRASTOR_MODULE 1
#undef  CONFIG_SCSI_DEBUG
#define CONFIG_SCSI_DEBUG_MODULE 1

/*
 * Network device support
 */
#define CONFIG_NETDEVICES 1
#undef  CONFIG_ARCNET
#define CONFIG_ARCNET_MODULE 1
#undef  CONFIG_ARCNET_ETH
#undef  CONFIG_ARCNET_1051
#undef  CONFIG_ARCNET_COM90xx
#define CONFIG_ARCNET_COM90xx_MODULE 1
#undef  CONFIG_ARCNET_COM90xxIO
#define CONFIG_ARCNET_COM90xxIO_MODULE 1
#undef  CONFIG_ARCNET_RIM_I
#define CONFIG_ARCNET_RIM_I_MODULE 1
#undef  CONFIG_ARCNET_COM20020
#define CONFIG_ARCNET_COM20020_MODULE 1
#define CONFIG_DUMMY 1
#undef  CONFIG_EQUALIZER
#define CONFIG_EQUALIZER_MODULE 1
#undef  CONFIG_ETHERTAP
#define CONFIG_ETHERTAP_MODULE 1
#define CONFIG_NET_ETHERNET 1
#define CONFIG_NET_VENDOR_3COM 1
#undef  CONFIG_EL1
#define CONFIG_EL1_MODULE 1
#undef  CONFIG_EL2
#define CONFIG_EL2_MODULE 1
#undef  CONFIG_ELPLUS
#define CONFIG_ELPLUS_MODULE 1
#undef  CONFIG_EL16
#define CONFIG_EL16_MODULE 1
#undef  CONFIG_EL3
#define CONFIG_EL3_MODULE 1
#undef  CONFIG_3C515
#define CONFIG_3C515_MODULE 1
#undef  CONFIG_VORTEX
#define CONFIG_VORTEX_MODULE 1
#undef  CONFIG_LANCE
#define CONFIG_LANCE_MODULE 1
#define CONFIG_NET_VENDOR_SMC 1
#undef  CONFIG_WD80x3
#define CONFIG_WD80x3_MODULE 1
#undef  CONFIG_ULTRA
#define CONFIG_ULTRA_MODULE 1
#undef  CONFIG_ULTRA32
#define CONFIG_ULTRA32_MODULE 1
#undef  CONFIG_SMC9194
#define CONFIG_SMC9194_MODULE 1
#define CONFIG_NET_VENDOR_RACAL 1
#undef  CONFIG_NI5010
#define CONFIG_NI5010_MODULE 1
#undef  CONFIG_NI52
#define CONFIG_NI52_MODULE 1
#undef  CONFIG_NI65
#define CONFIG_NI65_MODULE 1
#undef  CONFIG_RTL8139
#define CONFIG_RTL8139_MODULE 1
#undef  CONFIG_YELLOWFIN
#define CONFIG_YELLOWFIN_MODULE 1
#undef  CONFIG_ACENIC
#define CONFIG_ACENIC_MODULE 1
#define CONFIG_NET_ISA 1
#undef  CONFIG_AT1700
#define CONFIG_AT1700_MODULE 1
#undef  CONFIG_E2100
#define CONFIG_E2100_MODULE 1
#undef  CONFIG_DEPCA
#define CONFIG_DEPCA_MODULE 1
#undef  CONFIG_EWRK3
#define CONFIG_EWRK3_MODULE 1
#undef  CONFIG_EEXPRESS
#define CONFIG_EEXPRESS_MODULE 1
#undef  CONFIG_EEXPRESS_PRO
#define CONFIG_EEXPRESS_PRO_MODULE 1
#undef  CONFIG_FMV18X
#define CONFIG_FMV18X_MODULE 1
#undef  CONFIG_HPLAN_PLUS
#define CONFIG_HPLAN_PLUS_MODULE 1
#undef  CONFIG_HPLAN
#define CONFIG_HPLAN_MODULE 1
#undef  CONFIG_HP100
#define CONFIG_HP100_MODULE 1
#undef  CONFIG_ETH16I
#define CONFIG_ETH16I_MODULE 1
#undef  CONFIG_NE2000
#define CONFIG_NE2000_MODULE 1
#undef  CONFIG_SEEQ8005
#undef  CONFIG_SK_G16
#define CONFIG_NET_EISA 1
#undef  CONFIG_PCNET32
#define CONFIG_PCNET32_MODULE 1
#undef  CONFIG_AC3200
#define CONFIG_AC3200_MODULE 1
#undef  CONFIG_APRICOT
#define CONFIG_APRICOT_MODULE 1
#undef  CONFIG_CS89x0
#define CONFIG_CS89x0_MODULE 1
#undef  CONFIG_DE4X5
#define CONFIG_DE4X5_MODULE 1
#undef  CONFIG_DEC_ELCP
#define CONFIG_DEC_ELCP_MODULE 1
#undef  CONFIG_DGRS
#define CONFIG_DGRS_MODULE 1
#undef  CONFIG_EEXPRESS_PRO100
#define CONFIG_EEXPRESS_PRO100_MODULE 1
#undef  CONFIG_LNE390
#define CONFIG_LNE390_MODULE 1
#undef  CONFIG_NE3210
#define CONFIG_NE3210_MODULE 1
#undef  CONFIG_NE2K_PCI
#define CONFIG_NE2K_PCI_MODULE 1
#undef  CONFIG_TLAN
#define CONFIG_TLAN_MODULE 1
#undef  CONFIG_VIA_RHINE
#define CONFIG_VIA_RHINE_MODULE 1
#undef  CONFIG_ES3210
#define CONFIG_ES3210_MODULE 1
#undef  CONFIG_EPIC100
#define CONFIG_EPIC100_MODULE 1
#undef  CONFIG_ZNET
#define CONFIG_NET_POCKET 1
#undef  CONFIG_ATP
#undef  CONFIG_DE600
#define CONFIG_DE600_MODULE 1
#undef  CONFIG_DE620
#define CONFIG_DE620_MODULE 1
#undef  CONFIG_FDDI
#undef  CONFIG_HIPPI
#undef  CONFIG_DLCI
#define CONFIG_DLCI_MODULE 1
#define CONFIG_DLCI_COUNT (24)
#define CONFIG_DLCI_MAX (8)
#undef  CONFIG_SDLA
#define CONFIG_SDLA_MODULE 1
#undef  CONFIG_LTPC
#define CONFIG_LTPC_MODULE 1
#undef  CONFIG_COPS
#define CONFIG_COPS_MODULE 1
#undef  CONFIG_COPS_DAYNA
#undef  CONFIG_COPS_TANGENT
#undef  CONFIG_IPDDP
#define CONFIG_IPDDP_MODULE 1
#undef  CONFIG_IPDDP_ENCAP
#undef  CONFIG_IPDDP_DECAP
#undef  CONFIG_PLIP
#define CONFIG_PLIP_MODULE 1
#undef  CONFIG_PPP
#define CONFIG_PPP_MODULE 1

/*
 * CCP compressors for PPP are only built as modules.
 */
#undef  CONFIG_SLIP
#define CONFIG_SLIP_MODULE 1
#define CONFIG_SLIP_COMPRESSED 1
#define CONFIG_SLIP_SMART 1
#undef  CONFIG_SLIP_MODE_SLIP6
#define CONFIG_NET_RADIO 1
#undef  CONFIG_STRIP
#define CONFIG_STRIP_MODULE 1
#undef  CONFIG_WAVELAN
#define CONFIG_WAVELAN_MODULE 1
#define CONFIG_TR 1
#undef  CONFIG_IBMTR
#define CONFIG_IBMTR_MODULE 1
#undef  CONFIG_SKTR
#define CONFIG_SKTR_MODULE 1
#undef  CONFIG_SHAPER
#define CONFIG_SHAPER_MODULE 1
#undef  CONFIG_HOSTESS_SV11
#define CONFIG_HOSTESS_SV11_MODULE 1
#undef  CONFIG_COSA
#define CONFIG_COSA_MODULE 1
#undef  CONFIG_RCPCI
#define CONFIG_RCPCI_MODULE 1
#define CONFIG_WAN_DRIVERS 1
#undef  CONFIG_VENDOR_SANGOMA
#define CONFIG_VENDOR_SANGOMA_MODULE 1
#define CONFIG_WANPIPE_CARDS (1)
#define CONFIG_WANPIPE_X25 1
#define CONFIG_WANPIPE_FR 1
#define CONFIG_WANPIPE_PPP 1
#undef  CONFIG_LAPBETHER
#define CONFIG_LAPBETHER_MODULE 1
#undef  CONFIG_X25_ASY
#define CONFIG_X25_ASY_MODULE 1

/*
 * Amateur Radio support
 */
#define CONFIG_HAMRADIO 1

/*
 * Packet Radio protocols
 */
#undef  CONFIG_AX25
#define CONFIG_AX25_MODULE 1
#define CONFIG_AX25_DAMA_SLAVE 1
#undef  CONFIG_NETROM
#define CONFIG_NETROM_MODULE 1
#undef  CONFIG_ROSE
#define CONFIG_ROSE_MODULE 1

/*
 * AX.25 network device drivers
 */
#undef  CONFIG_MKISS
#define CONFIG_MKISS_MODULE 1
#undef  CONFIG_6PACK
#define CONFIG_6PACK_MODULE 1
#undef  CONFIG_BPQETHER
#define CONFIG_BPQETHER_MODULE 1
#undef  CONFIG_DMASCC
#define CONFIG_DMASCC_MODULE 1
#undef  CONFIG_SCC
#define CONFIG_SCC_MODULE 1
#define CONFIG_SCC_DELAY 1
#define CONFIG_SCC_TRXECHO 1
#undef  CONFIG_BAYCOM_SER_FDX
#define CONFIG_BAYCOM_SER_FDX_MODULE 1
#undef  CONFIG_BAYCOM_SER_HDX
#define CONFIG_BAYCOM_SER_HDX_MODULE 1
#undef  CONFIG_BAYCOM_PAR
#define CONFIG_BAYCOM_PAR_MODULE 1
#undef  CONFIG_BAYCOM_EPP
#define CONFIG_BAYCOM_EPP_MODULE 1
#undef  CONFIG_SOUNDMODEM
#define CONFIG_SOUNDMODEM_MODULE 1
#define CONFIG_SOUNDMODEM_SBC 1
#define CONFIG_SOUNDMODEM_WSS 1
#define CONFIG_SOUNDMODEM_AFSK1200 1
#define CONFIG_SOUNDMODEM_AFSK2400_7 1
#define CONFIG_SOUNDMODEM_AFSK2400_8 1
#define CONFIG_SOUNDMODEM_AFSK2666 1
#define CONFIG_SOUNDMODEM_HAPN4800 1
#define CONFIG_SOUNDMODEM_PSK4800 1
#define CONFIG_SOUNDMODEM_FSK9600 1

/*
 * Misc. hamradio protocols
 */
#undef  CONFIG_HFMODEM
#define CONFIG_HFMODEM_MODULE 1
#define CONFIG_HFMODEM_SBC 1
#define CONFIG_HFMODEM_WSS 1

/*
 * IrDA subsystem support
 */
#undef  CONFIG_IRDA
#define CONFIG_IRDA_MODULE 1

/*
 * IrDA protocols
 */
#undef  CONFIG_IRLAN
#define CONFIG_IRLAN_MODULE 1
#undef  CONFIG_IRCOMM
#define CONFIG_IRCOMM_MODULE 1
#undef  CONFIG_IRLPT
#define CONFIG_IRLPT_MODULE 1
#undef  CONFIG_IRLPT_CLIENT
#define CONFIG_IRLPT_CLIENT_MODULE 1
#undef  CONFIG_IRLPT_SERVER
#define CONFIG_IRLPT_SERVER_MODULE 1
#undef  CONFIG_IRDA_OPTIONS
#define CONFIG_IRDA_COMPRESSION 1

/*
 *    IrDA compressors
 */
#undef  CONFIG_IRDA_DEFLATE
#define CONFIG_IRDA_DEFLATE_MODULE 1

/*
 * Infrared-port device drivers
 */
#undef  CONFIG_IRTTY_SIR
#define CONFIG_IRTTY_SIR_MODULE 1

/*
 *    Dongle support
 */
#define CONFIG_DONGLE 1
#undef  CONFIG_ESI_DONGLE
#define CONFIG_ESI_DONGLE_MODULE 1
#undef  CONFIG_ACTISYS_DONGLE
#define CONFIG_ACTISYS_DONGLE_MODULE 1
#undef  CONFIG_TEKRAM_DONGLE
#define CONFIG_TEKRAM_DONGLE_MODULE 1
#undef  CONFIG_GIRBIL_DONGLE
#define CONFIG_GIRBIL_DONGLE_MODULE 1
#undef  CONFIG_NSC_FIR
#define CONFIG_NSC_FIR_MODULE 1
#undef  CONFIG_WINBOND_FIR
#define CONFIG_WINBOND_FIR_MODULE 1
#undef  CONFIG_SHARP_FIR
#define CONFIG_SHARP_FIR_MODULE 1

/*
 * ISDN subsystem
 */
#undef  CONFIG_ISDN
#define CONFIG_ISDN_MODULE 1
#define CONFIG_ISDN_PPP 1
#define CONFIG_ISDN_PPP_VJ 1
#define CONFIG_ISDN_MPP 1
#define CONFIG_ISDN_AUDIO 1
#define CONFIG_ISDN_X25 1
#undef  CONFIG_ISDN_DRV_ICN
#define CONFIG_ISDN_DRV_ICN_MODULE 1
#undef  CONFIG_ISDN_DRV_LOOP
#define CONFIG_ISDN_DRV_LOOP_MODULE 1
#undef  CONFIG_ISDN_DRV_PCBIT
#define CONFIG_ISDN_DRV_PCBIT_MODULE 1
#undef  CONFIG_ISDN_DRV_HISAX
#define CONFIG_ISDN_DRV_HISAX_MODULE 1
#define CONFIG_HISAX_EURO 1
#define CONFIG_DE_AOC 1
#define CONFIG_HISAX_ML 1
#define CONFIG_HISAX_1TR6 1
#define CONFIG_HISAX_16_0 1
#define CONFIG_HISAX_16_3 1
#define CONFIG_HISAX_TELES3C 1
#define CONFIG_HISAX_AVM_A1 1
#define CONFIG_HISAX_ELSA 1
#define CONFIG_HISAX_IX1MICROR2 1
#define CONFIG_HISAX_DIEHLDIVA 1
#define CONFIG_HISAX_ASUSCOM 1
#define CONFIG_HISAX_TELEINT 1
#define CONFIG_HISAX_SEDLBAUER 1
#define CONFIG_HISAX_SPORTSTER 1
#define CONFIG_HISAX_MIC 1
#define CONFIG_HISAX_NETJET 1
#define CONFIG_HISAX_NICCY 1
#undef  CONFIG_ISDN_DRV_SC
#define CONFIG_ISDN_DRV_SC_MODULE 1
#undef  CONFIG_ISDN_DRV_ACT2000
#define CONFIG_ISDN_DRV_ACT2000_MODULE 1
#undef  CONFIG_ISDN_DRV_AVMB1
#define CONFIG_ISDN_DRV_AVMB1_MODULE 1
#define CONFIG_ISDN_DRV_AVMB1_VERBOSE_REASON 1

/*
 * Old CD-ROM drivers (not SCSI, not IDE)
 */
#define CONFIG_CD_NO_IDESCSI 1
#undef  CONFIG_AZTCD
#define CONFIG_AZTCD_MODULE 1
#undef  CONFIG_GSCD
#define CONFIG_GSCD_MODULE 1
#undef  CONFIG_SBPCD
#define CONFIG_SBPCD_MODULE 1
#undef  CONFIG_MCD
#define CONFIG_MCD_MODULE 1
#define CONFIG_MCD_IRQ (11)
#define CONFIG_MCD_BASE 0x300
#undef  CONFIG_MCDX
#define CONFIG_MCDX_MODULE 1
#undef  CONFIG_OPTCD
#define CONFIG_OPTCD_MODULE 1
#undef  CONFIG_CM206
#define CONFIG_CM206_MODULE 1
#undef  CONFIG_SJCD
#define CONFIG_SJCD_MODULE 1
#undef  CONFIG_ISP16_CDI
#define CONFIG_ISP16_CDI_MODULE 1
#undef  CONFIG_CDU31A
#define CONFIG_CDU31A_MODULE 1
#undef  CONFIG_CDU535
#define CONFIG_CDU535_MODULE 1

/*
 * Character devices
 */
#define CONFIG_VT 1
#define CONFIG_VT_CONSOLE 1
#define CONFIG_SERIAL 1
#define CONFIG_SERIAL_CONSOLE 1
#define CONFIG_SERIAL_EXTENDED 1
#define CONFIG_SERIAL_MANY_PORTS 1
#define CONFIG_SERIAL_SHARE_IRQ 1
#undef  CONFIG_SERIAL_DETECT_IRQ
#define CONFIG_SERIAL_MULTIPORT 1
#define CONFIG_HUB6 1
#define CONFIG_SERIAL_NONSTANDARD 1
#undef  CONFIG_ROCKETPORT
#define CONFIG_ROCKETPORT_MODULE 1
#undef  CONFIG_DIGIEPCA
#define CONFIG_DIGIEPCA_MODULE 1
#undef  CONFIG_CYCLADES
#define CONFIG_CYCLADES_MODULE 1
#define CONFIG_STALDRV 1
#undef  CONFIG_STALLION
#define CONFIG_STALLION_MODULE 1
#undef  CONFIG_ISTALLION
#define CONFIG_ISTALLION_MODULE 1
#undef  CONFIG_RISCOM8
#define CONFIG_RISCOM8_MODULE 1
#undef  CONFIG_SPECIALIX
#define CONFIG_SPECIALIX_MODULE 1
#undef  CONFIG_SPECIALIX_RTSCTS
#undef  CONFIG_ESPSERIAL
#define CONFIG_ESPSERIAL_MODULE 1
#undef  CONFIG_ISI
#define CONFIG_ISI_MODULE 1
#undef  CONFIG_SYNCLINK
#define CONFIG_SYNCLINK_MODULE 1
#undef  CONFIG_N_HDLC
#define CONFIG_N_HDLC_MODULE 1
#undef  CONFIG_UNIX98_PTYS
#undef  CONFIG_PRINTER
#define CONFIG_PRINTER_MODULE 1
#define CONFIG_PRINTER_READBACK 1
#define CONFIG_MOUSE 1

/*
 * Mice
 */
#undef  CONFIG_ATIXL_BUSMOUSE
#define CONFIG_ATIXL_BUSMOUSE_MODULE 1
#undef  CONFIG_BUSMOUSE
#define CONFIG_BUSMOUSE_MODULE 1
#undef  CONFIG_MS_BUSMOUSE
#define CONFIG_MS_BUSMOUSE_MODULE 1
#define CONFIG_PSMOUSE 1
#undef  CONFIG_82C710_MOUSE
#define CONFIG_82C710_MOUSE_MODULE 1
#undef  CONFIG_PC110_PAD
#define CONFIG_PC110_PAD_MODULE 1
#undef  CONFIG_QIC02_TAPE
#define CONFIG_QIC02_TAPE_MODULE 1
#define CONFIG_QIC02_DYNCONF 1

/*
 *    Setting runtime QIC-02 configuration is done with qic02conf
 */

/*
 *    from the tpqic02-support package.  It is available at
 */

/*
 *    metalab.unc.edu or ftp://titus.cfw.com/pub/Linux/util/
 */
#define CONFIG_WATCHDOG 1

/*
 * Watchdog Cards
 */
#undef  CONFIG_WATCHDOG_NOWAYOUT
#undef  CONFIG_WDT
#define CONFIG_WDT_MODULE 1
#define CONFIG_WDT_501 1
#define CONFIG_WDT_501_FAN 1
#undef  CONFIG_SOFT_WATCHDOG
#define CONFIG_SOFT_WATCHDOG_MODULE 1
#undef  CONFIG_PCWATCHDOG
#define CONFIG_PCWATCHDOG_MODULE 1
#undef  CONFIG_ACQUIRE_WDT
#define CONFIG_ACQUIRE_WDT_MODULE 1
#undef  CONFIG_NVRAM
#define CONFIG_RTC 1

/*
 * Video For Linux
 */
#undef  CONFIG_VIDEO_DEV
#define CONFIG_VIDEO_DEV_MODULE 1
#undef  CONFIG_RADIO_RTRACK
#define CONFIG_RADIO_RTRACK_MODULE 1
#undef  CONFIG_RADIO_RTRACK2
#define CONFIG_RADIO_RTRACK2_MODULE 1
#undef  CONFIG_RADIO_AZTECH
#define CONFIG_RADIO_AZTECH_MODULE 1
#undef  CONFIG_RADIO_CADET
#define CONFIG_RADIO_CADET_MODULE 1
#undef  CONFIG_RADIO_MIROPCM20
#define CONFIG_RADIO_MIROPCM20_MODULE 1
#undef  CONFIG_RADIO_GEMTEK
#define CONFIG_RADIO_GEMTEK_MODULE 1
#undef  CONFIG_VIDEO_BT848
#define CONFIG_VIDEO_BT848_MODULE 1
#undef  CONFIG_VIDEO_BWQCAM
#define CONFIG_VIDEO_BWQCAM_MODULE 1
#undef  CONFIG_VIDEO_CQCAM
#define CONFIG_VIDEO_CQCAM_MODULE 1
#undef  CONFIG_VIDEO_PMS
#define CONFIG_VIDEO_PMS_MODULE 1
#undef  CONFIG_VIDEO_SAA5249
#define CONFIG_VIDEO_SAA5249_MODULE 1
#undef  CONFIG_RADIO_SF16FMI
#define CONFIG_RADIO_SF16FMI_MODULE 1
#undef  CONFIG_RADIO_TYPHOON
#define CONFIG_RADIO_TYPHOON_MODULE 1
#define CONFIG_RADIO_TYPHOON_PROC_FS 1
#undef  CONFIG_RADIO_ZOLTRIX
#define CONFIG_RADIO_ZOLTRIX_MODULE 1

/*
 * Joystick support
 */
#undef  CONFIG_JOYSTICK
#define CONFIG_JOYSTICK_MODULE 1
#undef  CONFIG_JOY_ANALOG
#define CONFIG_JOY_ANALOG_MODULE 1
#undef  CONFIG_JOY_ASSASIN
#define CONFIG_JOY_ASSASIN_MODULE 1
#undef  CONFIG_JOY_GRAVIS
#define CONFIG_JOY_GRAVIS_MODULE 1
#undef  CONFIG_JOY_LOGITECH
#define CONFIG_JOY_LOGITECH_MODULE 1
#undef  CONFIG_JOY_SIDEWINDER
#define CONFIG_JOY_SIDEWINDER_MODULE 1
#undef  CONFIG_JOY_THRUSTMASTER
#define CONFIG_JOY_THRUSTMASTER_MODULE 1
#undef  CONFIG_JOY_LIGHTNING
#define CONFIG_JOY_LIGHTNING_MODULE 1
#undef  CONFIG_JOY_CONSOLE
#define CONFIG_JOY_CONSOLE_MODULE 1
#undef  CONFIG_JOY_DB9
#define CONFIG_JOY_DB9_MODULE 1
#undef  CONFIG_JOY_TURBOGRAFX
#define CONFIG_JOY_TURBOGRAFX_MODULE 1
#undef  CONFIG_DTLK
#define CONFIG_DTLK_MODULE 1

/*
 * Ftape, the floppy tape device driver
 */
#undef  CONFIG_FTAPE
#define CONFIG_FTAPE_MODULE 1
#undef  CONFIG_ZFTAPE
#define CONFIG_ZFTAPE_MODULE 1
#define CONFIG_ZFT_DFLT_BLK_SZ (10240)

/*
 * The compressor will be built as a module only!
 */
#undef  CONFIG_ZFT_COMPRESSOR
#define CONFIG_ZFT_COMPRESSOR_MODULE 1
#define CONFIG_FT_NR_BUFFERS (3)
#undef  CONFIG_FT_PROC_FS
#define CONFIG_FT_NORMAL_DEBUG 1
#undef  CONFIG_FT_FULL_DEBUG
#undef  CONFIG_FT_NO_TRACE
#undef  CONFIG_FT_NO_TRACE_AT_ALL

/*
 * Hardware configuration
 */
#define CONFIG_FT_STD_FDC 1
#undef  CONFIG_FT_MACH2
#undef  CONFIG_FT_PROBE_FC10
#undef  CONFIG_FT_ALT_FDC
#define CONFIG_FT_FDC_THR (8)
#define CONFIG_FT_FDC_MAX_RATE (2000)

/*
 * ONLY for DEC Alpha architectures
 */
#define CONFIG_FT_ALPHA_CLOCK (0)

/*
 * Filesystems
 */
#define CONFIG_QUOTA 1
#undef  CONFIG_AUTOFS_FS
#define CONFIG_AUTOFS_FS_MODULE 1
#undef  CONFIG_ADFS_FS
#define CONFIG_ADFS_FS_MODULE 1
#undef  CONFIG_AFFS_FS
#define CONFIG_AFFS_FS_MODULE 1
#undef  CONFIG_HFS_FS
#define CONFIG_HFS_FS_MODULE 1
#define CONFIG_FAT_FS 1
#define CONFIG_MSDOS_FS 1
#define CONFIG_UMSDOS_FS 1
#define CONFIG_VFAT_FS 1
#define CONFIG_ISO9660_FS 1
#define CONFIG_JOLIET 1
#define CONFIG_MINIX_FS 1
#undef  CONFIG_NTFS_FS
#define CONFIG_NTFS_FS_MODULE 1
#undef  CONFIG_NTFS_RW
#undef  CONFIG_HPFS_FS
#define CONFIG_HPFS_FS_MODULE 1
#define CONFIG_PROC_FS 1
#undef  CONFIG_QNX4FS_FS
#define CONFIG_QNX4FS_FS_MODULE 1
#undef  CONFIG_QNX4FS_RW
#undef  CONFIG_ROMFS_FS
#define CONFIG_ROMFS_FS_MODULE 1
#define CONFIG_EXT2_FS 1
#undef  CONFIG_SYSV_FS
#define CONFIG_SYSV_FS_MODULE 1
#undef  CONFIG_UFS_FS
#define CONFIG_UFS_FS_MODULE 1
#undef  CONFIG_UFS_FS_WRITE

/*
 * Network File Systems
 */
#undef  CONFIG_CODA_FS
#define CONFIG_CODA_FS_MODULE 1
#define CONFIG_NFS_FS 1
#undef  CONFIG_NFSD
#define CONFIG_NFSD_MODULE 1
#undef  CONFIG_NFSD_SUN
#define CONFIG_SUNRPC 1
#define CONFIG_LOCKD 1
#undef  CONFIG_SMB_FS
#define CONFIG_SMB_FS_MODULE 1
#define CONFIG_SMB_WIN95 1
#undef  CONFIG_NCP_FS
#define CONFIG_NCP_FS_MODULE 1
#undef  CONFIG_NCPFS_PACKET_SIGNING
#undef  CONFIG_NCPFS_IOCTL_LOCKING
#define CONFIG_NCPFS_STRONG 1
#define CONFIG_NCPFS_NFS_NS 1
#define CONFIG_NCPFS_OS2_NS 1
#define CONFIG_NCPFS_MOUNT_SUBDIR 1

/*
 * Partition Types
 */
#define CONFIG_BSD_DISKLABEL 1
#define CONFIG_MAC_PARTITION 1
#define CONFIG_SMD_DISKLABEL 1
#define CONFIG_SOLARIS_X86_PARTITION 1
#define CONFIG_UNIXWARE_DISKLABEL 1
#define CONFIG_AMIGA_PARTITION 1
#define CONFIG_NLS 1

/*
 * Native Language Support
 */
#define CONFIG_NLS_CODEPAGE_437 1
#undef  CONFIG_NLS_CODEPAGE_737
#define CONFIG_NLS_CODEPAGE_737_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_775
#define CONFIG_NLS_CODEPAGE_775_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_850
#define CONFIG_NLS_CODEPAGE_850_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_852
#define CONFIG_NLS_CODEPAGE_852_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_855
#define CONFIG_NLS_CODEPAGE_855_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_857
#define CONFIG_NLS_CODEPAGE_857_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_860
#define CONFIG_NLS_CODEPAGE_860_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_861
#define CONFIG_NLS_CODEPAGE_861_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_862
#define CONFIG_NLS_CODEPAGE_862_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_863
#define CONFIG_NLS_CODEPAGE_863_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_864
#define CONFIG_NLS_CODEPAGE_864_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_865
#define CONFIG_NLS_CODEPAGE_865_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_866
#define CONFIG_NLS_CODEPAGE_866_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_869
#define CONFIG_NLS_CODEPAGE_869_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_874
#define CONFIG_NLS_CODEPAGE_874_MODULE 1
#define CONFIG_NLS_ISO8859_1 1
#undef  CONFIG_NLS_ISO8859_2
#define CONFIG_NLS_ISO8859_2_MODULE 1
#undef  CONFIG_NLS_ISO8859_3
#define CONFIG_NLS_ISO8859_3_MODULE 1
#undef  CONFIG_NLS_ISO8859_4
#define CONFIG_NLS_ISO8859_4_MODULE 1
#undef  CONFIG_NLS_ISO8859_5
#define CONFIG_NLS_ISO8859_5_MODULE 1
#undef  CONFIG_NLS_ISO8859_6
#define CONFIG_NLS_ISO8859_6_MODULE 1
#undef  CONFIG_NLS_ISO8859_7
#define CONFIG_NLS_ISO8859_7_MODULE 1
#undef  CONFIG_NLS_ISO8859_8
#define CONFIG_NLS_ISO8859_8_MODULE 1
#undef  CONFIG_NLS_ISO8859_9
#define CONFIG_NLS_ISO8859_9_MODULE 1
#undef  CONFIG_NLS_ISO8859_15
#define CONFIG_NLS_ISO8859_15_MODULE 1
#undef  CONFIG_NLS_KOI8_R
#define CONFIG_NLS_KOI8_R_MODULE 1

/*
 * Console drivers
 */
#define CONFIG_VGA_CONSOLE 1
#define CONFIG_VIDEO_SELECT 1
#undef  CONFIG_MDA_CONSOLE
#define CONFIG_FB 1
#define CONFIG_DUMMY_CONSOLE 1
#undef  CONFIG_FB_PM2
#define CONFIG_FB_VESA 1
#define CONFIG_VIDEO_SELECT 1
#undef  CONFIG_FB_MATROX
#undef  CONFIG_FB_ATY
#undef  CONFIG_FB_VIRTUAL
#undef  CONFIG_FBCON_ADVANCED
#define CONFIG_FBCON_CFB8 1
#define CONFIG_FBCON_CFB16 1
#define CONFIG_FBCON_CFB24 1
#define CONFIG_FBCON_CFB32 1
#undef  CONFIG_FBCON_FONTWIDTH8_ONLY
#undef  CONFIG_FBCON_FONTS
#define CONFIG_FONT_8x8 1
#define CONFIG_FONT_8x16 1

/*
 * Sound
 */
#undef  CONFIG_SOUND
#define CONFIG_SOUND_MODULE 1
#undef  CONFIG_SOUND_ES1370
#define CONFIG_SOUND_ES1370_MODULE 1
#undef  CONFIG_SOUND_ES1371
#define CONFIG_SOUND_ES1371_MODULE 1
#undef  CONFIG_SOUND_SONICVIBES
#define CONFIG_SOUND_SONICVIBES_MODULE 1
#undef  CONFIG_SOUND_MSNDCLAS
#define CONFIG_SOUND_MSNDCLAS_MODULE 1
#undef  CONFIG_MSNDCLAS_HAVE_BOOT
#define CONFIG_MSNDCLAS_INIT_FILE "/etc/sound/msndinit.bin"
#define CONFIG_MSNDCLAS_PERM_FILE "/etc/sound/msndperm.bin"
#undef  CONFIG_SOUND_MSNDPIN
#define CONFIG_SOUND_MSNDPIN_MODULE 1
#undef  CONFIG_MSNDPIN_HAVE_BOOT
#define CONFIG_MSNDPIN_INIT_FILE "/etc/sound/pndspini.bin"
#define CONFIG_MSNDPIN_PERM_FILE "/etc/sound/pndsperm.bin"
#undef  CONFIG_SOUND_OSS
#define CONFIG_SOUND_OSS_MODULE 1
#undef  CONFIG_SOUND_PAS
#define CONFIG_SOUND_PAS_MODULE 1
#undef  CONFIG_SOUND_SB
#define CONFIG_SOUND_SB_MODULE 1
#undef  CONFIG_SOUND_ADLIB
#define CONFIG_SOUND_ADLIB_MODULE 1
#undef  CONFIG_SOUND_GUS
#define CONFIG_SOUND_GUS_MODULE 1
#undef  CONFIG_GUS16
#undef  CONFIG_GUSMAX
#undef  CONFIG_SOUND_MPU401
#define CONFIG_SOUND_MPU401_MODULE 1
#undef  CONFIG_SOUND_PSS
#define CONFIG_SOUND_PSS_MODULE 1
#undef  CONFIG_PSS_MIXER
#undef  CONFIG_SOUND_MSS
#define CONFIG_SOUND_MSS_MODULE 1
#undef  CONFIG_SOUND_SSCAPE
#define CONFIG_SOUND_SSCAPE_MODULE 1
#undef  CONFIG_SOUND_TRIX
#define CONFIG_SOUND_TRIX_MODULE 1
#undef  CONFIG_SOUND_MAD16
#define CONFIG_SOUND_MAD16_MODULE 1
#undef  CONFIG_MAD16_OLDCARD
#undef  CONFIG_SOUND_WAVEFRONT
#define CONFIG_SOUND_WAVEFRONT_MODULE 1
#undef  CONFIG_SOUND_CS4232
#define CONFIG_SOUND_CS4232_MODULE 1
#undef  CONFIG_SOUND_OPL3SA2
#define CONFIG_SOUND_OPL3SA2_MODULE 1
#undef  CONFIG_SOUND_MAUI
#define CONFIG_SOUND_MAUI_MODULE 1
#undef  CONFIG_SOUND_SGALAXY
#define CONFIG_SOUND_SGALAXY_MODULE 1
#undef  CONFIG_SOUND_AD1816
#define CONFIG_SOUND_AD1816_MODULE 1
#undef  CONFIG_SOUND_OPL3SA1
#define CONFIG_SOUND_OPL3SA1_MODULE 1
#undef  CONFIG_SOUND_SOFTOSS
#define CONFIG_SOUND_SOFTOSS_MODULE 1
#undef  CONFIG_SOUND_YM3812
#define CONFIG_SOUND_YM3812_MODULE 1
#undef  CONFIG_SOUND_VMIDI
#define CONFIG_SOUND_VMIDI_MODULE 1
#undef  CONFIG_SOUND_UART6850
#define CONFIG_SOUND_UART6850_MODULE 1

/*
 * Additional low level sound drivers
 */
#define CONFIG_LOWLEVEL_SOUND 1
#undef  CONFIG_ACI_MIXER
#define CONFIG_ACI_MIXER_MODULE 1
#undef  CONFIG_AWE32_SYNTH
#define CONFIG_AWE32_SYNTH_MODULE 1
#undef  CONFIG_AEDSP16
#define CONFIG_AEDSP16_MODULE 1
#define CONFIG_AEDSP16_BASE 0x220
#define CONFIG_MPU_BASE 0x330

/*
 * SC-6600 Audio Cards have no jumper switches at all
 */
#define CONFIG_SC6600 1

/*
 * SC-6600 specific configuration
 */
#define CONFIG_SC6600_JOY 1
#define CONFIG_SC6600_CDROM (4)
#define CONFIG_SC6600_CDROMBASE 0x0
#define CONFIG_AEDSP16_SBPRO 1

/*
 * Audio Excel DSP 16 [Sound Blaster Pro]
 */
#define CONFIG_AEDSP16_BASE 0x220
#define CONFIG_AEDSP16_SB_IRQ (5)
#define CONFIG_AEDSP16_SB_DMA (0)
#define CONFIG_AEDSP16_MPU401 1

/*
 * Audio Excel DSP 16 [MPU-401]
 */
#define CONFIG_AEDSP16_MPU_IRQ (0)

/*
 * Kernel hacking
 */
#undef  CONFIG_MAGIC_SYSRQ
